<?php

namespace App\Http\Controllers;

use Clarkeash\Doorman\Exceptions\{DoormanException,ExpiredInviteCode};
use Clarkeash\Doorman\Facades\Doorman;
use Illuminate\Contracts\View\Factory;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\View\View;
use Barryvdh\Snappy\Facades\SnappyPdf as PDF;

use App\Models\{Invoice,Service};
use App\User;

class UserHomeController extends Controller
{
	public function __construct()
	{
		// Route protection is in routes.web
		// $this->middleware('auth');
	}

	/**
	 * Logged in users home page
	 *
	 * @return Factory|View
	 */
	public function home(User $o=NULL): View
	{
		if ($o)
			return View('u.home',['o'=>$o]);

		// If User was null, then test and see what type of logged on user we have
		$o = Auth::user();

		switch (Auth::user()->role()) {
			case 'customer':
				return View('u.home',['o'=>$o]);

			case 'reseller':
			case 'wholesaler':
				return View('r.home',['o'=>$o]);

			default:
				abort(500,'Unknown role: '.$o->role());
		}
	}

	/**
	 * Render a specific invoice for the user
	 *
	 * @param Invoice $o
	 * @return View
	 */
	public function invoice(Invoice $o): View
	{
		return View('u.invoice.home',['o'=>$o]);
	}

	/**
	 * Return the invoice in PDF format, ready to download
	 *
	 * @param Invoice $o
	 * @return mixed
	 */
	public function invoice_pdf(Invoice $o)
	{
		return PDF::loadView('u.invoice.home',['o'=>$o])->stream(sprintf('%s.pdf',$o->invoice_account_id));
	}

	/**
	 * Enable the user to down an invoice by providing a link in email
	 *
	 * @param Invoice $o
	 * @param string  $code
	 * @return \Illuminate\Http\RedirectResponse|mixed
	 */
	public function invoice_pdf_email(Invoice $o,string $code)
	{
		try {
			Doorman::redeem($code,$o->account->user->email);

		} catch (ExpiredInviteCode $e) {
			Log::alert(sprintf('User is using an expired token for invoice [%s] using [%s]',$o->id,$code));

			return redirect()->to('/login');

		} catch (DoormanException $e) {
			Log::alert(sprintf('An attempt to read invoice id [%s] using [%s]',$o->id,$code));

			abort(404);
		}

		return $this->invoice_pdf($o);
	}

	/**
	 * Helper to redirect to the old site, when functions are not available in this one.
	 *
	 * @param $type
	 * @param $action
	 * @param $id
	 * @return void
	 * @deprecated @todo Remove once all functions added
	 */
	public function oldsite($type,$action,$id)
	{
		abort(307,sprintf('http://www.graytech.net.au/u/%s/%s/%s',$type,$action,$id));
	}

	/**
	 * Return details on the users service
	 *
	 * @param Service $o
	 * @return View
	 */
	public function service(Service $o): View
	{
		return View('u.service.home',['o'=>$o]);
	}

	/**
	 * Progress the order to the next stage
	 *
	 * @note: Route Middleware protects this path
	 * @param Service $o
	 * @param string  $status
	 * @return \Illuminate\Http\RedirectResponse
	 */
	public function service_progress(Service $o,string $status)
	{
		return redirect()->to($o->action($status) ?: url('u/service',$o->id));
	}
}